/*

	BO2 Style Buildables Script by JR-Imagine

*/

//	      _ _____      _____                       _            
//	     | |  __ \    |_   _|                     (_)           
//	     | | |__) |_____| |  _ __ ___   __ _  __ _ _ _ __   ___ 
//	 _   | |  _  /______| | | '_ ` _ \ / _` |/ _` | | '_ \ / _ \
//	| |__| | | \ \     _| |_| | | | | | (_| | (_| | | | | |  __/
//	 \____/|_|  \_\   |_____|_| |_| |_|\__,_|\__, |_|_| |_|\___|
//	                                          __/ |             
//	                                         |___/  

#include common_scripts\utility;
#include maps\_utility;
#include maps\_zombiemode_utility;
#include maps\_hud_util;
#include maps\trem_hintstrings;

// Init buildables
main()
{
	// Init Vars
	players = get_players();
	for(i=0;i<players.size;i++)
	{
		players[i].IsCarryingPart = false; // Player is not carrying a part, so set this to false
		players[i].CarriedPart = undefined; // Player is not carrying a part, so set this to undefined
	}

	level.BuildableTime = 4; // Time it takes to build something

	thread buildables_parts_init();
	thread buildables_benches_init();
}

// Precache models
PrecacheBuildables()
{
	// Add all xmodels for your parts here
	PrecacheModel( "XMODEL_NAME_HERE" );
	
	PrecacheShader( "MATERIAL_NAME_OF_PART_SHADER" );
}

// Init parts
buildables_parts_init()
{
	// This is how you add a part, no extra scripting required
	// CHANGE THESE
	buildables_part( "XMODEL_NAME_HERE", "STRING_USED_TO_STORE_PART_NAME_VARIABLE", "MATERIAL_NAME_OF_PART_SHADER" );
}

// Add part
buildables_part( model, part, shader )
{
	locations = GetEntArray( "buildable_location", "targetname" ); // Get all locations that are not occupied yet (this is why we don't thread this function)
	wait .05; // Wait just to make sure... Doesn't really matter, just bugged out on me sometimes and this fixed it
	location = locations[ RandomInt( locations.size ) ]; // Pick a random location out of all available locations

	thread buildables_part_add( model, part, shader, location ); // Spawn part
	wait 1; // Wait longer than it takes to spawn a part (.5 seconds) to make sure the location occupied by this part will be deleted before spawning a new part
}

buildables_part_add( xmodel, part, shader, location )
{
	wait .5;

	model = Spawn("script_model", location.origin); // Spawn a script_model at the location picked by buildables_part()
	model SetModel( xmodel ); // Change the model to the xmodel defined by buildables_part()
	location Delete(); // Delete the location script_origin

	trig = Spawn("trigger_radius", model.origin, 48, 48, 32); // Spawn a trigger_radius at the origin of the model we just spawned
	trig LinkTo( model );
	trig UseTriggerRequireLookAt();
	trig setCursorHint("HINT_NOICON");
	trig _setHintString("Press &&1 to pick up part");

	// Picking up parts logic
	while( isDefined( trig ) )
	{
		players = get_players(); // Get all player entities
		for(i=0;i<players.size;i++)
		{
			// If these conditions are true, then continue the logic
			if( players[i] isTouching( trig ) && players[i] UseButtonPressed() && !players[i] maps\_laststand::player_is_in_laststand() && !players[i].being_revived )
			{
				// If player is already carrying a part, drop it
				if( players[i].IsCarryingPart == true )
					players[i] drop_current_part();

				players[i].IsCarryingPart = true; // Player is now carrying a part, so set this to true
				players[i].CarriedPart = part; // Set the part the player is currently holding to the string defined by buildables_part()

				// If a shader is set for this part, then create a hud element for it
				if( isDefined(shader) )
				{
					players[i].PartHud = create_simple_hud(players[i]);
					players[i].PartHud.foreground = true; 
					players[i].PartHud.sort = 1; 
					players[i].PartHud.hidewheninmenu = false; 
					players[i].PartHud.alignX = "left"; 
					players[i].PartHud.alignY = "bottom";
					players[i].PartHud.horzAlign = "left"; 
					players[i].PartHud.vertAlign = "bottom";
					players[i].PartHud.x = 10; 
					players[i].PartHud.y = -115; 
					players[i].PartHud.alpha = 1;
					players[i].PartHud SetShader( shader, 40, 40 );
				}

				model Delete(); // Delete the model
				trig Delete(); // Delete the trigger
			}
		}

		wait .05;
	}
}

// Drop part
drop_current_part()
{
	if( self.IsCarryingPart ) // Make sure we're not trying to drop a part when the player doesn't have one
	{
		model = undefined;
		shader = undefined;

		switch( self.CarriedPart )
		{
			// CHANGE THESE
			case "STRING_USED_TO_STORE_PART_NAME_VARIABLE":
				model = "XMODEL_NAME_HERE";
				shader = "MATERIAL_NAME_OF_PART_SHADER";
				break;
		}

		location = Spawn("script_origin", self.origin); // Spawn a script_origin at the player's origin to use as a location to spawn the part we dropped
		thread buildables_part_add( model, self.CarriedPart, shader, location ); // Spawn the part we dropped

		// Inventory stuff again
		self.IsCarryingPart = false;
		self.CarriedPart = undefined;

		self.PartHud Destroy(); // Destroy HUD
	}
}

// Init benches
buildables_benches_init()
{
	// Soulcrusher
	level.WORKBENCH_VARIABLE = getEnt("TARGETNAME_SET_IN_RADIANT_FOR_WORKBENCH","targetname"); // CHANGE THESE
	thread buildables_bench( level.WORKBENCH_VARIABLE, "STRING_USED_TO_STORE_PART_NAME_VARIABLE", "STRING_USED_TO_STORE_PART_NAME_VARIABLE", "STRING_USED_TO_STORE_PART_NAME_VARIABLE" ); // Define up to 3 strings used for parts required for the buildable to be built
}

// Add buildables bench
buildables_bench(bench, part1, part2, part3)
{
	// Vars
	// They're not on the workbench yet... so...
	part1_isAdded = false;
	part2_isAdded = false;
	part3_isAdded = false;
	isBuilt = false; // Obviously it's not built yet...

	// Check if parts are defined, if not, set Added var to true (incase you don't want 3 parts)
	if( !isDefined(part1) )
		part1_isAdded = true;
	if( !isDefined(part2) )
		part2_isAdded = true;
	if( !isDefined(part3) )
		part3_isAdded = true;

	// Spawn a trigger on the workbench
	trig = Spawn("trigger_radius", bench.origin, 60, 60, 60);
	trig linkto(bench);
	trig setCursorHint("HINT_NOICON");
	trig _setHintString("Press &&1 to add part");

	// Loop untill all parts are added
	while( !isBuilt )
	{
		players = get_players();
		for(i=0;i<players.size;i++)
		{
			if( players[i] isTouching( trig ) && players[i] isOnGround() && ( isDefined(players[i].CarriedPart) && ( players[i].CarriedPart == part1 || players[i].CarriedPart == part2 || players[i].CarriedPart == part3 ) ) && players[i] UseButtonPressed() )
				{
					if( (players[i].CarriedPart == part1 && part1_isAdded == false) || (players[i].CarriedPart == part2 && part2_isAdded == false) || (players[i].CarriedPart == part3 && part3_isAdded == false) )
					{
						players[i].BuildableProgressBar = players[i] CreatePrimaryProgressBar();
						players[i].BuildableProgressBar.color = ( .5, 1, 1 );
						players[i].BuildableProgressBar UpdateBar( 0.01, 1 / level.BuildableTime );

						// players[i] PlaySound( "zmb_building" ); // Uncomment incase you have a sound
						players[i] thread do_knuckle_crack();

						time = 0;

						while( players[i] UseButtonPressed() && players[i] isTouching( trig ) && players[i] isOnGround() && !players[i] maps\_laststand::player_is_in_laststand() && !players[i].being_revived && time < level.BuildableTime )
						{
							time += .05;
							wait .05;
						}

						players[i] StopSounds();
						players[i] notify( "buildable_complete" );
						players[i].BuildableProgressBar destroyElem();

						if( time >= level.BuildableTime )
						{
							// Add part
							if(isDefined(part1) && players[i].CarriedPart == part1)
								part1_isAdded = true;
							if(isDefined(part2) && players[i].CarriedPart == part2)
								part2_isAdded = true;
							if(isDefined(part3) && players[i].CarriedPart == part3)
								part3_isAdded = true;

							// Empty inventory and clear hud
							players[i].IsCarryingPart = false;
							players[i].CarriedPart = undefined;
							if( isDefined(players[i].PartHud) )
								players[i].PartHud Destroy();

							// Check if all parts are added, if so, thread item's functionality
							if( part1_isAdded && part2_isAdded && part3_isAdded )
							{
								// players[i] PlaySound("zmb_build_completed"); // All parts are added, build complete sound plays (Uncomment incase you have a sound)
								thread buildables_bench_item(trig, bench);
								isBuilt = true;
							}
							//else // If not all parts are added, play sound to add a part
								//players[i] PlaySound("zmb_build_add_piece"); // Uncomment incase you have a sound
						}
					}
				}
		}
		wait .05;
	}
}

// Whatever the item is supposed to do
buildables_bench_item(trig, bench)
{
	wait 1;

	// Do whatever you need to do here
}

// Knuckle crack animation
do_knuckle_crack()
{
	self DisableOffhandWeapons();
	self DisableWeaponCycling();

	self AllowLean(false);
	self AllowAds(false);
	self AllowSprint(false);
	self AllowProne(false);		
	self AllowMelee(false);
	
	if( self GetStance() == "prone" )
		self SetStance("crouch");

	gun = self GetCurrentWeapon();
	self GiveWeapon( "zombie_knuckle_crack" );
	self SwitchToWeapon( "zombie_knuckle_crack" );
	self waittill_any( "fake_death", "death", "player_downed", "weapon_change_complete", "buildable_complete" );

	self EnableOffhandWeapons();
	self EnableWeaponCycling();
	self TakeWeapon( "zombie_knuckle_crack" );
	self SwitchToWeapon( gun );

	self AllowLean(true);
	self AllowAds(true);
	self AllowSprint(true);
	self AllowProne(true);
	self AllowMelee(true);
	self notify("knuckle_crack_done");
}